@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 - 2010 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 - 2011 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  *
  * This library is free software; you can redistribute it and/or
@@ -53,25 +53,24 @@ static void gdaui_data_cell_renderer_textual_set_property  (GObject *object,
 							    GParamSpec *pspec);
 static void gdaui_data_cell_renderer_textual_get_size   (GtkCellRenderer          *cell,
 							 GtkWidget                *widget,
-							 GdkRectangle             *cell_area,
+							 const GdkRectangle       *cell_area,
 							 gint                     *x_offset,
 							 gint                     *y_offset,
 							 gint                     *width,
 							 gint                     *height);
 static void gdaui_data_cell_renderer_textual_render     (GtkCellRenderer          *cell,
-							 GdkWindow                *window,
+							 cairo_t                  *cr,
 							 GtkWidget                *widget,
-							 GdkRectangle             *background_area,
-							 GdkRectangle             *cell_area,
-							 GdkRectangle             *expose_area,
+							 const GdkRectangle       *background_area,
+							 const GdkRectangle       *cell_area,
 							 GtkCellRendererState      flags);
 
 static GtkCellEditable *gdaui_data_cell_renderer_textual_start_editing (GtkCellRenderer      *cell,
 									GdkEvent             *event,
 									GtkWidget            *widget,
 									const gchar          *path,
-									GdkRectangle         *background_area,
-									GdkRectangle         *cell_area,
+									const GdkRectangle   *background_area,
+									const GdkRectangle   *cell_area,
 									GtkCellRendererState  flags);
 
 enum {
@@ -152,7 +151,7 @@ gdaui_data_cell_renderer_textual_init (GdauiDataCellRendererTextual *datacell)
 {
 	datacell->priv = g_new0 (GdauiDataCellRendererTextualPrivate, 1);
 	datacell->priv->dh = NULL;
-	datacell->priv->type = G_TYPE_INVALID;
+	datacell->priv->type = GDA_TYPE_NULL;
 	datacell->priv->type_forced = FALSE;
 	datacell->priv->value = NULL;
 	datacell->priv->options = NULL;
@@ -430,7 +429,7 @@ gdaui_data_cell_renderer_textual_set_property (GObject *object,
 				if (G_VALUE_TYPE (gval) != datacell->priv->type) {
 					if (!datacell->priv->type_forced) {
 						datacell->priv->type_forced = TRUE;
-						if (datacell->priv->type != G_TYPE_INVALID)
+						if (datacell->priv->type != GDA_TYPE_NULL)
 							g_warning (_("Data cell renderer's specified type (%s) "
 								     "differs from actual "
 								     "value to display type (%s)"),
@@ -452,7 +451,7 @@ gdaui_data_cell_renderer_textual_set_property (GObject *object,
 				if (! OPTIMIZE)
 					datacell->priv->value = gda_value_copy (gval);
 
-				if (!datacell->priv->dh && (datacell->priv->type != G_TYPE_INVALID))
+				if (!datacell->priv->dh && (datacell->priv->type != GDA_TYPE_NULL))
 					datacell->priv->dh = g_object_ref (gda_data_handler_get_default (datacell->priv->type));
 
 				if (datacell->priv->dh) {
@@ -572,7 +571,7 @@ gdaui_data_cell_renderer_textual_set_property (GObject *object,
 
 /**
  * gdaui_data_cell_renderer_textual_new:
- * @dh: (allow-none): a #GdaDataHandler object, or %NULL
+ * @dh: a #GdaDataHandler object, or %NULL
  * @type: the #GType being edited
  * @options: options as a string
  *
@@ -603,7 +602,7 @@ gdaui_data_cell_renderer_textual_new (GdaDataHandler *dh, GType type, const gcha
 static void
 gdaui_data_cell_renderer_textual_get_size (GtkCellRenderer *cell,
 					   GtkWidget       *widget,
-					   GdkRectangle    *cell_area,
+					   const GdkRectangle *cell_area,
 					   gint            *x_offset,
 					   gint            *y_offset,
 					   gint            *width,
@@ -615,29 +614,30 @@ gdaui_data_cell_renderer_textual_get_size (GtkCellRenderer *cell,
 
 static void
 gdaui_data_cell_renderer_textual_render (GtkCellRenderer      *cell,
-					 GdkWindow            *window,
+					 cairo_t              *cr,
 					 GtkWidget            *widget,
-					 GdkRectangle         *background_area,
-					 GdkRectangle         *cell_area,
-					 GdkRectangle         *expose_area,
+					 const GdkRectangle   *background_area,
+					 const GdkRectangle   *cell_area,
 					 GtkCellRendererState  flags)
 
 {
 	GdauiDataCellRendererTextual *datacell = (GdauiDataCellRendererTextual*) cell;
 	GtkCellRendererClass *text_class = g_type_class_peek (GTK_TYPE_CELL_RENDERER_TEXT);
-	(text_class->render) (cell, window, widget, background_area, cell_area, expose_area, flags);
+	(text_class->render) (cell, cr, widget, background_area, cell_area, flags);
 
 	if (datacell->priv->to_be_deleted) {
-		cairo_t *cr;
-		cr = gdk_cairo_create (window);
-		cairo_set_source_rgba (cr, 0., 0., 0., 1.);
-		cairo_rectangle (cr, cell_area->x, cell_area->y + cell_area->height / 2. - .5,
-				 cell_area->width, 1.);
-		cairo_fill (cr);
-		cairo_destroy (cr);
+		GtkStyleContext *style_context = gtk_widget_get_style_context (widget);
+		guint xpad;
+		g_object_get ((GObject*) cell, "xpad", &xpad, NULL);
+
+		gdouble y = cell_area->y + cell_area->height / 2.;
+		gtk_render_line (style_context,
+				 cr,
+				 cell_area->x + xpad, cell_area->x + cell_area->width - xpad,
+				 y, y);
 	}
 	if (datacell->priv->invalid)
-		gdaui_data_cell_renderer_draw_invalid_area (window, cell_area);
+		gdaui_data_cell_renderer_draw_invalid_area (cr, cell_area);
 }
 
 static void
@@ -687,8 +687,8 @@ gdaui_data_cell_renderer_textual_start_editing (GtkCellRenderer      *cell,
 						G_GNUC_UNUSED GdkEvent             *event,
 						G_GNUC_UNUSED GtkWidget            *widget,
 						const gchar          *path,
-						G_GNUC_UNUSED GdkRectangle         *background_area,
-						G_GNUC_UNUSED GdkRectangle         *cell_area,
+						G_GNUC_UNUSED const GdkRectangle *background_area,
+						G_GNUC_UNUSED const GdkRectangle *cell_area,
 						G_GNUC_UNUSED GtkCellRendererState  flags)
 {
 	GdauiDataCellRendererTextual *datacell;
