@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2010 David King <davidk@openismus.com>
- * Copyright (C) 2010 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2010 - 2011 Vivien Malerba <malerba@gnome-db.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,20 +18,20 @@
  * Boston, MA  02110-1301, USA.
  */
 #include "widget-embedder.h"
-#if GTK_CHECK_VERSION (2,20,0)
-#include <gdaui-decl.h>
-
 static void     widget_embedder_realize       (GtkWidget       *widget);
 static void     widget_embedder_unrealize     (GtkWidget       *widget);
-static void     widget_embedder_size_request  (GtkWidget       *widget,
-                                               GtkRequisition  *requisition);
+static void     widget_embedder_get_preferred_width  (GtkWidget *widget,
+						      gint      *minimum,
+						      gint      *natural);
+static void     widget_embedder_get_preferred_height (GtkWidget *widget,
+						      gint      *minimum,
+						      gint      *natural);
 static void     widget_embedder_size_allocate (GtkWidget       *widget,
                                                GtkAllocation   *allocation);
 static gboolean widget_embedder_damage        (GtkWidget       *widget,
                                                GdkEventExpose  *event);
-static gboolean widget_embedder_expose        (GtkWidget       *widget,
-                                               GdkEventExpose  *offscreen);
-
+static gboolean widget_embedder_draw          (GtkWidget       *widget,
+					       cairo_t         *cr);
 static void     widget_embedder_add           (GtkContainer    *container,
                                                GtkWidget       *child);
 static void     widget_embedder_remove        (GtkContainer    *container,
@@ -74,9 +74,10 @@ widget_embedder_class_init (WidgetEmbedderClass *klass)
 
 	widget_class->realize = widget_embedder_realize;
 	widget_class->unrealize = widget_embedder_unrealize;
-	widget_class->size_request = widget_embedder_size_request;
+	widget_class->get_preferred_width = widget_embedder_get_preferred_width;
+	widget_class->get_preferred_height = widget_embedder_get_preferred_height;
 	widget_class->size_allocate = widget_embedder_size_allocate;
-	widget_class->expose_event = widget_embedder_expose;
+	widget_class->draw = widget_embedder_draw;
 
 	g_signal_override_class_closure (g_signal_lookup ("damage-event", GTK_TYPE_WIDGET),
 					 WIDGET_EMBEDDER_TYPE,
@@ -94,10 +95,6 @@ widget_embedder_init (WidgetEmbedder *bin)
 {
 	gtk_widget_set_has_window (GTK_WIDGET (bin), TRUE);
 	bin->valid = TRUE;
-	bin->red = -1.;
-	bin->green = -1.;
-	bin->blue = -1.;
-	bin->alpha = -1.;
 }
 
 GtkWidget *
@@ -179,10 +176,9 @@ widget_embedder_realize (GtkWidget *widget)
 		| GDK_LEAVE_NOTIFY_MASK;
 
 	attributes.visual = gtk_widget_get_visual (widget);
-	attributes.colormap = gtk_widget_get_colormap (widget);
 	attributes.wclass = GDK_INPUT_OUTPUT;
 
-	attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
+	attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL;
 
 	GdkWindow *win;
 	win = gdk_window_new (gtk_widget_get_parent_window (widget),
@@ -304,7 +300,7 @@ widget_embedder_size_request (GtkWidget      *widget,
 	child_requisition.height = 0;
 
 	if (bin->child && gtk_widget_get_visible (bin->child))
-		gtk_widget_size_request (bin->child, &child_requisition);
+		gtk_widget_get_preferred_size (bin->child, &child_requisition, NULL);
 
 	guint border_width;
 	border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
@@ -312,6 +308,26 @@ widget_embedder_size_request (GtkWidget      *widget,
 	requisition->height = border_width * 2 + child_requisition.height;
 }
 
+static void
+widget_embedder_get_preferred_width (GtkWidget *widget,
+				     gint      *minimum,
+				     gint      *natural)
+{
+	GtkRequisition requisition;
+	widget_embedder_size_request (widget, &requisition);
+	*minimum = *natural = requisition.width;
+}
+
+static void
+widget_embedder_get_preferred_height (GtkWidget *widget,
+				      gint      *minimum,
+				      gint      *natural)
+{
+	GtkRequisition requisition;
+	widget_embedder_size_request (widget, &requisition);
+	*minimum = *natural = requisition.height;
+}
+
 static void
 widget_embedder_size_allocate (GtkWidget     *widget,
                                GtkAllocation *allocation)
@@ -337,8 +353,10 @@ widget_embedder_size_allocate (GtkWidget     *widget,
 	}
 
 	if (bin->child && gtk_widget_get_visible (bin->child)){
+		GtkRequisition child_requisition;
 		GtkAllocation child_allocation;
 		
+		gtk_widget_get_child_requisition (bin->child, &child_requisition);
 		child_allocation.x = 0;
 		child_allocation.y = 0;
 		child_allocation.height = h;
@@ -365,77 +383,37 @@ widget_embedder_damage (GtkWidget      *widget,
 	return TRUE;
 }
 
-void
-widget_embedder_set_ucolor (WidgetEmbedder *bin, gdouble red, gdouble green,
-			    gdouble blue, gdouble alpha)
-{
-	bin->red = red;
-	bin->green = green;
-	bin->blue = blue;
-	bin->alpha = alpha;
-	gtk_widget_queue_draw (GTK_WIDGET (bin));
-}
-
-
 static gboolean
-widget_embedder_expose (GtkWidget      *widget,
-                        GdkEventExpose *event)
+widget_embedder_draw (GtkWidget *widget, cairo_t *cr)
 {
 	WidgetEmbedder *bin = WIDGET_EMBEDDER (widget);
-	gint width, height;
-#define MARGIN 1.5
+	GdkWindow *window;
 
-	if (gtk_widget_is_drawable (widget)) {
-		GdkWindow *win;
-		win = gtk_widget_get_window (widget);
-		if (event->window == win) {
-			GdkPixmap *pixmap;
-			cairo_t *cr;
+	window = gtk_widget_get_window (widget);
+	if (gtk_cairo_should_draw_window (cr, window)) {
+		cairo_surface_t *surface;
+		GtkAllocation child_area;
+		
+		if (bin->child && gtk_widget_get_visible (bin->child)) {
+			surface = gdk_offscreen_window_get_surface (bin->offscreen_window);
+			gtk_widget_get_allocation (bin->child, &child_area);
+			cairo_set_source_surface (cr, surface, 0, 0);
+			cairo_paint (cr);
 
-			if (bin->child && gtk_widget_get_visible (bin->child)) {
-				GtkAllocation child_area;
-				pixmap = gdk_offscreen_window_get_pixmap (bin->offscreen_window);
-
-				gtk_widget_get_allocation (bin->child, &child_area);
-				cr = gdk_cairo_create (win);
-				
-				/* clip */
-				gdk_drawable_get_size (pixmap, &width, &height);
-				cairo_rectangle (cr, 0, 0, width, height);
-				cairo_clip (cr);
-
-				/* paint */
-				gdk_cairo_set_source_pixmap (cr, pixmap, 0, 0);
-				cairo_paint (cr);
-
-				if (! bin->valid) {
-					if ((bin->red >= 0.) && (bin->red <= 1.) &&
-					    (bin->green >= 0.) && (bin->green <= 1.) &&
-					    (bin->blue >= 0.) && (bin->blue <= 1.) &&
-					    (bin->alpha >= 0.) && (bin->alpha <= 1.))
-						cairo_set_source_rgba (cr, bin->red, bin->green,
-								       bin->blue, bin->alpha);
-					else
-						cairo_set_source_rgba (cr, GDAUI_COLOR_UNKNOWN_MASK);
-					cairo_rectangle (cr, child_area.x + MARGIN, child_area.y + MARGIN,
-							 child_area.width - 2. * MARGIN,
-							 child_area.height - 2. * MARGIN);
-					cairo_fill (cr);
-				}
-				cairo_destroy (cr);
+			if (! bin->valid) {
+				cairo_set_source_rgba (cr, .8, .1, .1, .2);
+				cairo_rectangle (cr, child_area.x, child_area.y,
+						 child_area.width, child_area.height);
+				cairo_fill (cr);
 			}
 		}
-		else if (event->window == bin->offscreen_window) {
-			gtk_paint_flat_box (gtk_widget_get_style (widget), event->window,
-					    GTK_STATE_NORMAL, GTK_SHADOW_NONE,
-					    &event->area, widget, "blah",
-					    0, 0, -1, -1);
-			
-			if (bin->child)
-				gtk_container_propagate_expose (GTK_CONTAINER (widget),
-								bin->child,
-								event);
-		}
+	}
+
+	if (gtk_cairo_should_draw_window (cr, bin->offscreen_window)) {
+		if (bin->child)
+			gtk_container_propagate_draw (GTK_CONTAINER (widget),
+						      bin->child,
+						      cr);
 	}
 
 	return FALSE;
@@ -454,5 +432,3 @@ widget_embedder_set_valid (WidgetEmbedder *bin, gboolean valid)
 	bin->valid = valid;
 	gtk_widget_queue_draw (GTK_WIDGET (bin));
 }
-
-#endif
