@@ -138,12 +138,6 @@ config_dsn_changed_cb (G_GNUC_UNUSED GdaConfig *config, GdaDsnInfo *dsn, GdauiLo
 	}
 }
 
-static void
-auth_data_changed_cb (GdauiProviderAuthEditor *auth, GdauiLogin *login)
-{
-	g_signal_emit (login, gdaui_login_signals [CHANGED], 0, settings_are_valid (login));
-}
-
 static void
 gdaui_login_init (GdauiLogin *login, G_GNUC_UNUSED GdauiLoginClass *klass)
 {
@@ -156,6 +150,8 @@ gdaui_login_init (GdauiLogin *login, G_GNUC_UNUSED GdauiLoginClass *klass)
 	/* Init the properties*/
 	login->priv->mode = GDA_UI_LOGIN_ENABLE_CONTROL_CENTRE_MODE;
 	memset (&(login->priv->dsn_info), 0, sizeof (GdaDsnInfo));
+
+	gtk_orientable_set_orientation (GTK_ORIENTABLE (login), GTK_ORIENTATION_VERTICAL);
 	
 	/* catch DSN definition changes */
 	GdaConfig *conf = gda_config_get ();
@@ -221,8 +217,6 @@ gdaui_login_init (GdauiLogin *login, G_GNUC_UNUSED GdauiLoginClass *klass)
 	login->priv->auth_widget = _gdaui_provider_auth_editor_new (NULL);
 	gtk_table_attach_defaults (GTK_TABLE (table), login->priv->auth_widget, 1, 3, 3, 4);
 	gtk_widget_show (login->priv->auth_widget);
-	g_signal_connect (login->priv->auth_widget, "changed",
-			  G_CALLBACK (auth_data_changed_cb), login);
 
 	prov_entry_changed_cb (GDAUI_PROVIDER_SELECTOR (login->priv->prov_selector), login);
 }
@@ -336,7 +330,7 @@ gdaui_login_get_type (void)
 			(GInstanceInitFunc) gdaui_login_init,
 			0
 		};
-		type = g_type_register_static (GTK_TYPE_VBOX, "GdauiLogin", &info, 0);
+		type = g_type_register_static (GTK_TYPE_BOX, "GdauiLogin", &info, 0);
 	}
 	return type;
 }
@@ -389,40 +383,45 @@ radio_button_use_dsn_toggled_cb (GtkToggleButton *button, GdauiLogin *login)
 static void
 run_cc_cb (G_GNUC_UNUSED GtkButton *button, GdauiLogin *login)
 {
-	char *argv[2];
+	GAppInfo *appinfo;
+	GdkAppLaunchContext *context;
+	GdkScreen *screen;
 	gboolean sresult;
 	GError *lerror = NULL;
+	gchar *cmd;
 	
 #ifdef G_OS_WIN32
 #define EXENAME "gda-control-center-" GDA_ABI_VERSION ".exe"
 #else
 #define EXENAME "gda-control-center-" GDA_ABI_VERSION
 #endif
-	/* run gnome-database-properties dictig tool */
-	argv[0] = gda_gbr_get_file_path (GDA_BIN_DIR, (char *) EXENAME, NULL);
-	argv[1] = NULL;
-        
-	sresult = gdk_spawn_on_screen (gtk_widget_get_screen (GTK_WIDGET (login)),
-				       NULL, argv, NULL, 0,
-				       NULL, NULL, NULL, &lerror);
-	if (!sresult && lerror && (lerror->domain == G_SPAWN_ERROR) && (lerror->code == G_SPAWN_ERROR_NOENT)) {
-		g_error_free (lerror);
-		lerror = NULL;
-		g_free (argv [0]);
-		argv[0] = g_strdup ((char *) EXENAME);
-		sresult = gdk_spawn_on_screen (gtk_widget_get_screen (GTK_WIDGET (login)),
-					       NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
-					       NULL, NULL, NULL, &lerror);
+	/* run gnome-database-properties tool */
+	cmd = gda_gbr_get_file_path (GDA_BIN_DIR, (char *) EXENAME, NULL);
+	appinfo = g_app_info_create_from_commandline (cmd,
+						      "Gda Control center",
+						      G_APP_INFO_CREATE_NONE,
+						      NULL);
+	g_free (cmd);
+
+	screen = gtk_widget_get_screen (GTK_WIDGET (login));
+	context = gdk_display_get_app_launch_context (gdk_screen_get_display (screen));
+	gdk_app_launch_context_set_screen (context, screen);
+	sresult = g_app_info_launch (appinfo, NULL, G_APP_LAUNCH_CONTEXT (context), NULL);
+	if (! sresult) {
+		g_object_unref (appinfo);
+		appinfo = g_app_info_create_from_commandline (EXENAME,
+							      "Gda Control center",
+							      G_APP_INFO_CREATE_NONE,
+							      NULL);
+		sresult = g_app_info_launch (appinfo, NULL, G_APP_LAUNCH_CONTEXT (context), &lerror);
 	}
-	g_free (argv [0]);
+	g_object_unref (context);
+	g_object_unref (appinfo);
+
 	if (!sresult) {
 		GtkWidget *msgdialog;
 		GtkWidget *toplevel = gtk_widget_get_toplevel (GTK_WIDGET (login));
-#if GTK_CHECK_VERSION(2,18,0)
 		if (!gtk_widget_is_toplevel (toplevel))
-#else
-		if (!GTK_WIDGET_TOPLEVEL (toplevel))
-#endif
 			toplevel = NULL;
 		msgdialog = gtk_message_dialog_new_with_markup (GTK_WINDOW (toplevel), GTK_DIALOG_MODAL,
 								GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
@@ -467,9 +466,6 @@ dsn_entry_changed_cb (GdauiDsnSelector *sel, GdauiLogin *login)
 	_gdaui_provider_spec_editor_set_specs (GDAUI_PROVIDER_SPEC_EDITOR (login->priv->cnc_params_editor),
 					       info ? info->cnc_string : NULL);
 
-	if (login->priv->auth_widget)
-		gtk_widget_grab_focus (login->priv->auth_widget);
-
 	g_signal_emit (login, gdaui_login_signals [CHANGED], 0, settings_are_valid (login));
 }
 
@@ -489,9 +485,6 @@ prov_entry_changed_cb (GdauiProviderSelector *sel, GdauiLogin *login)
 	_gdaui_provider_auth_editor_set_provider (GDAUI_PROVIDER_AUTH_EDITOR (login->priv->auth_widget),
 						 prov);
 
-	if (login->priv->auth_widget)
-		gtk_widget_grab_focus (login->priv->auth_widget);
-
 	g_signal_emit (login, gdaui_login_signals [CHANGED], 0, settings_are_valid (login));
 }
 
@@ -503,7 +496,7 @@ cnc_params_editor_changed_cb (G_GNUC_UNUSED GdauiProviderSpecEditor *editor, Gda
 
 /**
  * gdaui_login_new:
- * @dsn: (allow-none): a data source name, or %NULL
+ * @dsn: a data source name, or %NULL
  *
  * Creates a new login widget which enables the user to specify connection parameters.
  *
@@ -626,7 +619,7 @@ gdaui_login_get_connection_information (GdauiLogin *login)
 /**
  * gdaui_login_set_dsn:
  * @login: a #GdauiLogin object
- * @dsn: (allow-none): a data source name, or %NULL
+ * @dsn: a data source name, or %NULL
  *
  * Changes the information displayed in @login, to represent @dsn.
  * If @login's mode has %GDA_UI_LOGIN_HIDE_DSN_SELECTION_MODE, then
