@@ -174,7 +174,7 @@ gdaui_server_operation_get_type (void)
 			0
 		};
 
-		type = g_type_register_static (GTK_TYPE_VBOX, "GdauiServerOperation", &info, 0);
+		type = g_type_register_static (GTK_TYPE_BOX, "GdauiServerOperation", &info, 0);
 	}
 
 	return type;
@@ -215,6 +215,8 @@ gdaui_server_operation_init (GdauiServerOperation * wid)
 	wid->priv->glade = NULL;
 #endif
 	wid->priv->opt_header = FALSE;
+
+	gtk_orientable_set_orientation (GTK_ORIENTABLE (wid), GTK_ORIENTATION_VERTICAL);
 }
 
 
@@ -476,7 +478,7 @@ fill_create_widget (GdauiServerOperation *form, const gchar *path, gchar **secti
 		winfo = gdaui_data_proxy_info_new (GDAUI_DATA_PROXY (grid),
 						   GDAUI_DATA_PROXY_INFO_ROW_MODIFY_BUTTONS);
 
-		box = gtk_vbox_new (FALSE, 0);
+		box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 		gtk_box_pack_start (GTK_BOX (box), plwid, TRUE, TRUE, 0);
 		gtk_widget_show (plwid);
 
@@ -767,7 +769,7 @@ gdaui_server_operation_fill (GdauiServerOperation *form)
 				gtk_label_set_markup (GTK_LABEL (label), section_str);
 				g_free (section_str);
 
-				hbox = gtk_hbox_new (FALSE, 0); /* HIG */
+				hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0); /* HIG */
 				gtk_widget_show (hbox);
 				lab = gtk_label_new ("    ");
 				gtk_box_pack_start (GTK_BOX (hbox), lab, FALSE, FALSE, 0);
@@ -940,7 +942,7 @@ sequence_item_added_cb (GdaServerOperation *op, const gchar *seq_path, gint item
 			}
 
 			/* move children DOWN if necessary and change the "_index" property */
-			if (top_attach >= item_index) {
+			if (top_attach >= (guint)item_index) {
 				struct MoveChild *mc;
 				gint index;
 
@@ -1016,7 +1018,7 @@ sequence_item_remove_cb (GdaServerOperation *op, const gchar *seq_path, gint ite
 			guint top_attach;
 			gtk_container_child_get (GTK_CONTAINER (table), child,
 						 "top-attach", &top_attach, NULL);
-			if (top_attach == item_index) {
+			if (top_attach == (guint)item_index) {
 				gtk_widget_destroy (child);
 				g_list_free (children);
 				children = gtk_container_get_children (GTK_CONTAINER (table));
@@ -1046,7 +1048,7 @@ sequence_item_remove_cb (GdaServerOperation *op, const gchar *seq_path, gint ite
 			}
 
 			/* move widgets UP if necessary and change the "_index" property */
-			if (top_attach > item_index) {
+			if (top_attach > (guint)item_index) {
 				struct MoveChild *mc;
 				gint index;
 
@@ -1162,10 +1164,10 @@ create_table_fields_array_create_widget (GdauiServerOperation *form, const gchar
 	info_node = gda_server_operation_get_node_info (form->priv->op, path);
 	g_assert (info_node->type == GDA_SERVER_OPERATION_NODE_DATA_MODEL);
 
-	hlayout = gtk_hpaned_new ();
+	hlayout = gtk_paned_new (GTK_ORIENTATION_HORIZONTAL);
 
 	/* form for field properties */
-	box = gtk_vbox_new (FALSE, 0);
+	box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 	gtk_paned_pack2 (GTK_PANED (hlayout), box, TRUE, TRUE);
 
 	label = gtk_label_new (_("<b>Field properties:</b>"));
@@ -1184,7 +1186,7 @@ create_table_fields_array_create_widget (GdauiServerOperation *form, const gchar
 	gtk_widget_show_all (box);
 
 	/* grid for field names */
-	box = gtk_vbox_new (FALSE, 0);
+	box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 	gtk_paned_pack1 (GTK_PANED (hlayout), box, TRUE, TRUE);
 
 	label = gtk_label_new (_("<b>Fields:</b>"));
