@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2008 Bas Driessen <bas.driessen@xobas.com>
  * Copyright (C) 2008 Murray Cumming <murrayc@murrayc.com>
- * Copyright (C) 2008 - 2010 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2008 - 2011 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  * Copyright (C) 2010 Jonh Wendell <jwendell@gnome.org>
  *
@@ -66,7 +66,6 @@ static GObjectClass  *parent_class = NULL;
 
 struct _GdaStatementPrivate {
 	GdaSqlStatement *internal_struct;
-	GType           *requested_types;
 };
 
 /* signals */
@@ -179,7 +178,6 @@ gda_statement_init (GdaStatement * stmt)
 {
 	stmt->priv = g_new0 (GdaStatementPrivate, 1);
 	stmt->priv->internal_struct = NULL;
-	stmt->priv->requested_types = NULL;
 }
 
 /**
@@ -198,7 +196,7 @@ gda_statement_new (void)
 	return GDA_STATEMENT (obj);
 }
 
-GdaSqlStatement *
+const GdaSqlStatement *
 _gda_statement_get_internal_struct (GdaStatement *stmt)
 {
 	return stmt->priv->internal_struct;
@@ -233,10 +231,6 @@ gda_statement_dispose (GObject *object)
 
 	stmt = GDA_STATEMENT (object);
 	if (stmt->priv) {
-		if (stmt->priv->requested_types) {
-			g_free (stmt->priv->requested_types);
-			stmt->priv->requested_types = NULL;
-		}
 		if (stmt->priv->internal_struct) {
 			gda_sql_statement_free (stmt->priv->internal_struct);
 			stmt->priv->internal_struct = NULL;
@@ -282,10 +276,6 @@ gda_statement_set_property (GObject *object,
 				gda_sql_statement_free (stmt->priv->internal_struct);
 				stmt->priv->internal_struct = NULL;
 			}
-			if (stmt->priv->requested_types) {
-				g_free (stmt->priv->requested_types);
-				stmt->priv->requested_types = NULL;
-			}
 			stmt->priv->internal_struct = gda_sql_statement_copy (g_value_get_pointer (value));
 			g_signal_emit (stmt, gda_statement_signals [RESET], 0);
 			break;
@@ -492,7 +482,7 @@ get_params_foreach_func (GdaSqlAnyPart *node, GdaSet **params, GError **error)
 	    (pspec = ((GdaSqlExpr*) node)->param_spec)) {
 		GdaHolder *h;
 
-		if (pspec->g_type == 0) {
+		if (pspec->g_type == GDA_TYPE_NULL) {
 			g_set_error (error, GDA_STATEMENT_ERROR, GDA_STATEMENT_PARAM_TYPE_ERROR,
 				     _("Could not determine GType for parameter '%s'"),
 				     pspec->name ? pspec->name : _("Unnamed"));
@@ -505,30 +495,7 @@ get_params_foreach_func (GdaSqlAnyPart *node, GdaSet **params, GError **error)
 			      "description", pspec->descr, NULL);
 		gda_holder_set_not_null (h, ! pspec->nullok);
 		if (((GdaSqlExpr*) node)->value) {
-			/* Expr's value is "SQL encoded" => we need to convert it to a real value */
-			GValue *evalue;
-			evalue = ((GdaSqlExpr*) node)->value;
-			if (G_VALUE_TYPE (evalue) == G_TYPE_STRING) {
-				GdaDataHandler *dh;
-				GValue *value;
-				dh = gda_data_handler_get_default (pspec->g_type);
-				if (!dh) {
-					g_set_error (error, GDA_STATEMENT_ERROR, GDA_STATEMENT_PARAM_TYPE_ERROR,
-						     _("Can't handle default value of type '%s'"),
-						     gda_g_type_to_string (pspec->g_type));
-					g_object_unref (h);
-					return FALSE;
-				}
-				value = gda_data_handler_get_value_from_sql (dh,
-									     g_value_get_string (evalue),
-									     pspec->g_type);
-				if (!value)
-					value = gda_value_new_default (g_value_get_string (evalue));
-				gda_holder_set_default_value (h, value);
-				gda_value_free (value);
-			}
-			else
-				gda_holder_set_default_value (h, ((GdaSqlExpr*) node)->value);
+			gda_holder_set_default_value (h, ((GdaSqlExpr*) node)->value);
 			gda_holder_set_value_to_default (h);
 		}
 		gda_set_add_holder (*params, h);
@@ -576,57 +543,6 @@ gda_statement_get_parameters (GdaStatement *stmt, GdaSet **out_params, GError **
 	return TRUE;
 }
 
-/*
- * _gda_statement_get_requested_types:
- * @stmt: a #GdaStatement
- *
- * Returns: a new #GType, suitable to use with gda_connection_statement_execute_select_full(), or %NULL
- */
-const GType *
-_gda_statement_get_requested_types (GdaStatement *stmt)
-{
-	if (! stmt->priv || ! stmt->priv->internal_struct)
-		return NULL;
-	if (stmt->priv->requested_types)
-		return stmt->priv->requested_types;
-	if (stmt->priv->internal_struct->stmt_type != GDA_SQL_STATEMENT_SELECT)
-		return NULL;
-
-	GdaSqlStatementSelect *selst;
-	GSList *list;
-	GArray *array = NULL;
- rewind:
-	selst = (GdaSqlStatementSelect*) stmt->priv->internal_struct->contents;
-	for (list = selst->expr_list; list; list = list->next) {
-		GdaSqlExpr *expr;
-		GType type = G_TYPE_INVALID;
-		expr = ((GdaSqlSelectField*) list->data)->expr;
-		if (expr->cast_as && *expr->cast_as)
-			type = gda_g_type_from_string (expr->cast_as);
-		if (array) {
-			if (type == G_TYPE_INVALID)
-				type = 0;
-			g_array_append_val (array, type);
-		}
-		else if (type != G_TYPE_INVALID) {
-			array = g_array_new (TRUE, FALSE, sizeof (GType));
-			goto rewind;
-		}
-	}
-	if (array) {
-		GType *retval;
-		guint len;
-		len = array->len;
-		retval = (GType*) g_array_free (array, FALSE);
-		retval [len] = G_TYPE_NONE;
-		stmt->priv->requested_types = retval;
-		return retval;
-	}
-	else
-		return NULL;
-}
-
-
 /*
  * SQL rendering
  */
@@ -653,7 +569,6 @@ static gchar *default_render_select_target (GdaSqlSelectTarget *target, GdaSqlRe
 static gchar *default_render_select_join (GdaSqlSelectJoin *join, GdaSqlRenderingContext *context, GError **error);
 static gchar *default_render_select_from (GdaSqlSelectFrom *from, GdaSqlRenderingContext *context, GError **error);
 static gchar *default_render_select_order (GdaSqlSelectOrder *order, GdaSqlRenderingContext *context, GError **error);
-static gchar *default_render_distinct (GdaSqlStatementSelect *select, GdaSqlRenderingContext *context, GError **error);
 
 /**
  * gda_statement_to_sql_real:
@@ -715,8 +630,6 @@ gda_statement_to_sql_real (GdaStatement *stmt, GdaSqlRenderingContext *context,
 		context->render_select_from = (GdaSqlRenderingFunc) default_render_select_from;
 	if (!context->render_select_order)
 		context->render_select_order = (GdaSqlRenderingFunc) default_render_select_order;
-	if (!context->render_distinct)
-		context->render_distinct = (GdaSqlRenderingFunc) default_render_distinct;
 
 	cinfo = gda_sql_statement_get_contents_infos (stmt->priv->internal_struct->stmt_type);
 	if (cinfo->check_structure_func && !cinfo->check_structure_func (GDA_SQL_ANY_PART (stmt->priv->internal_struct->contents), 
@@ -787,14 +700,9 @@ default_render_value (const GValue *value, GdaSqlRenderingContext *context, GErr
 			dh = gda_data_handler_get_default (G_VALUE_TYPE (value));
 
 		if (!dh) {
-			if (G_VALUE_TYPE (value) == GDA_TYPE_DEFAULT)
-				return g_strdup ("DEFAULT");
-			else {
-				g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
-					     _("No data handler for type '%s'"),
-					     g_type_name (G_VALUE_TYPE (value)));
-				return NULL;
-			}
+			g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
+				     _("No data handler for type '%s'"), g_type_name (G_VALUE_TYPE (value)));
+			return NULL;
 		}
 		return gda_data_handler_get_sql_from_value (dh, value);
 	}
@@ -1120,9 +1028,7 @@ default_render_compound (GdaSqlStatementCompound *stmt, GdaSqlRenderingContext *
 		case GDA_SQL_ANY_STMT_SELECT:
 			str = context->render_select (GDA_SQL_ANY_PART (sqlstmt->contents), context, error);
 			if (!str) goto err;
-			g_string_append_c (string, '(');
 			g_string_append (string, str);
-			g_string_append_c (string, ')');
 			g_free (str);
 			break;
 		case GDA_SQL_ANY_STMT_COMPOUND:
@@ -1147,36 +1053,6 @@ default_render_compound (GdaSqlStatementCompound *stmt, GdaSqlRenderingContext *
 	return NULL;	
 }
 
-static gchar *
-default_render_distinct (GdaSqlStatementSelect *stmt, GdaSqlRenderingContext *context, GError **error)
-{
-	gboolean pretty = context->flags & GDA_STATEMENT_SQL_PRETTY;
-	g_return_val_if_fail (stmt, NULL);
-	g_return_val_if_fail (GDA_SQL_ANY_PART (stmt)->type == GDA_SQL_ANY_STMT_SELECT, NULL);
-
-	if (stmt->distinct) {
-		GString *string;
-		string = g_string_new ("DISTINCT");
-		if (stmt->distinct_expr) {
-			gchar *str;
-			str = context->render_expr (stmt->distinct_expr, context, NULL, NULL, error);
-			if (!str) {
-				g_string_free (string, TRUE);
-				return NULL;
-			}
-			g_string_append (string, " ON (");
-			g_string_append (string, str);
-			g_string_append (string, ") ");
-			g_free (str);
-		}
-		if (pretty)
-			g_string_append_c (string, '\n');
-		return g_string_free (string, FALSE);
-	}
-	else
-		return NULL;
-}
-
 static gchar *
 default_render_select (GdaSqlStatementSelect *stmt, GdaSqlRenderingContext *context, GError **error)
 {
@@ -1191,11 +1067,17 @@ default_render_select (GdaSqlStatementSelect *stmt, GdaSqlRenderingContext *cont
 	string = g_string_new ("SELECT ");
 	/* distinct */
 	if (stmt->distinct) {
-		str = context->render_distinct (GDA_SQL_ANY_PART (stmt), context, error);
-		if (!str) goto err;
-		g_string_append (string, str);
-		g_string_append_c (string, ' ');
-		g_free (str);
+		g_string_append (string, "DISTINCT ");
+		if (stmt->distinct_expr) {
+			str = context->render_expr (stmt->distinct_expr, context, NULL, NULL, error);
+			if (!str) goto err;
+			g_string_append (string, "ON ");
+			g_string_append (string, str);
+			g_string_append_c (string, ' ');
+			g_free (str);
+		}
+		if (pretty)
+			g_string_append_c (string, '\n');
 	}
 	
 	/* selected expressions */
@@ -1411,7 +1293,7 @@ default_render_param_spec (GdaSqlParamSpec *pspec, GdaSqlExpr *expr, GdaSqlRende
 				     "%s", _("Unnamed parameter"));
 			goto err;
 		}
-		quoted_pname = gda_sql_identifier_add_quotes (pspec->name);
+		quoted_pname = gda_sql_identifier_force_quotes (pspec->name);
 
 		if (! (flag & (GDA_STATEMENT_SQL_PARAMS_LONG | GDA_STATEMENT_SQL_PARAMS_SHORT))) {
 			if (!expr->value || gda_value_is_null (expr->value) || strcmp (quoted_pname, pspec->name))
@@ -1422,10 +1304,7 @@ default_render_param_spec (GdaSqlParamSpec *pspec, GdaSqlExpr *expr, GdaSqlRende
 
 		if (flag & GDA_STATEMENT_SQL_PARAMS_LONG) {
 			if (expr->value) {
-				if (G_VALUE_TYPE (expr->value) == G_TYPE_STRING)
-					str = g_value_dup_string (expr->value);
-				else
-					str = context->render_value (expr->value, context, error);
+				str = context->render_value (expr->value, context, error);
 				if (!str) {
 					g_free (quoted_pname);
 					goto err;
@@ -1439,12 +1318,12 @@ default_render_param_spec (GdaSqlParamSpec *pspec, GdaSqlExpr *expr, GdaSqlRende
 			g_string_append (string, " /* ");
 			g_string_append_printf (string, "name:%s", quoted_pname);
 			if (pspec->g_type) {
-				str = gda_sql_identifier_add_quotes (gda_g_type_to_string (pspec->g_type));
+				str = gda_sql_identifier_force_quotes (gda_g_type_to_string (pspec->g_type));
 				g_string_append_printf (string, " type:%s", str);
 				g_free (str);
 			}
 			if (pspec->descr) {
-				str = gda_sql_identifier_add_quotes (pspec->descr);
+				str = gda_sql_identifier_force_quotes (pspec->descr);
 				g_string_append_printf (string, " descr:%s", str);
 				g_free (str);
 			}
@@ -1456,7 +1335,7 @@ default_render_param_spec (GdaSqlParamSpec *pspec, GdaSqlExpr *expr, GdaSqlRende
 		else {
 			g_string_append (string, "##");
 			g_string_append (string, pspec->name);
-			if (pspec->g_type != G_TYPE_INVALID) {
+			if (pspec->g_type != GDA_TYPE_NULL) {
 				g_string_append (string, "::");
 				g_string_append (string, gda_g_type_to_string (pspec->g_type));
 				if (pspec->nullok) 
@@ -1503,74 +1382,44 @@ default_render_expr (GdaSqlExpr *expr, GdaSqlRenderingContext *context, gboolean
 		if (!str) goto err;
 	}
 	else if (expr->value) {
-		if (G_VALUE_TYPE (expr->value) == G_TYPE_STRING) {
-			/* specific treatment for strings, see documentation about GdaSqlExpr's value attribute */
-			const gchar *vstr;
-			vstr = g_value_get_string (expr->value);
-			if (vstr) {
-				if (expr->value_is_ident) {
-					gchar **ids_array;
-					gint i;
-					GString *string = NULL;
-					GdaConnectionOptions cncoptions = 0;
-					if (context->cnc)
-						g_object_get (G_OBJECT (context->cnc), "options", &cncoptions, NULL);
-					ids_array = gda_sql_identifier_split (vstr);
-					if (!ids_array)
-						str = g_strdup (vstr);
-					else if (!(ids_array[0])) goto err;
-					else {
-						for (i = 0; ids_array[i]; i++) {
-							gchar *tmp;
-							if (!string)
-								string = g_string_new ("");
-							else
-								g_string_append_c (string, '.');
-							tmp = gda_sql_identifier_quote (ids_array[i], context->cnc,
-											context->provider, FALSE,
-											cncoptions & GDA_CONNECTION_OPTIONS_SQL_IDENTIFIERS_CASE_SENSITIVE);
-							g_string_append (string, tmp);
-							g_free (tmp);
-						}
-						g_strfreev (ids_array);
-						str = g_string_free (string, FALSE);
-					}
-				}
-				else {
-					/* we don't have an identifier */
-					if (!g_ascii_strcasecmp (vstr, "default")) {
-						if (is_default)
-							*is_default = TRUE;
-						str = g_strdup ("DEFAULT");
-					}
-					else
-						str = g_strdup (vstr);
-				}
-			}
+		if (expr->value_is_ident && (G_VALUE_TYPE (expr->value) == G_TYPE_STRING) &&
+		    g_value_get_string (expr->value)) {
+			gchar **ids_array;
+			gint i;
+			GString *string = NULL;
+			GdaConnectionOptions cncoptions = 0;
+			if (context->cnc)
+				g_object_get (G_OBJECT (context->cnc), "options", &cncoptions, NULL);
+
+			ids_array = gda_sql_identifier_split (g_value_get_string (expr->value));
+			if (!ids_array) 
+				str = g_value_dup_string (expr->value);
+			else if (!(ids_array[0])) goto err;
 			else {
-				str = g_strdup ("NULL");
-				if (is_null)
-					*is_null = TRUE;
+				for (i = 0; ids_array[i]; i++) {
+					gchar *tmp;
+					if (!string)
+						string = g_string_new ("");
+					else
+						g_string_append_c (string, '.');
+					tmp = gda_sql_identifier_quote (ids_array[i], context->cnc,
+									context->provider, FALSE,
+					   cncoptions & GDA_CONNECTION_OPTIONS_SQL_IDENTIFIERS_CASE_SENSITIVE);
+					g_string_append (string, tmp);
+					g_free (tmp);
+				}
+				g_strfreev (ids_array);
+				str = g_string_free (string, FALSE);
 			}
 		}
-		if (!str) {
-			/* use a GdaDataHandler to render the value as valid SQL */
-			GdaDataHandler *dh;
-			if (context->cnc) {
-				GdaServerProvider *prov;
-				prov = gda_connection_get_provider (context->cnc);
-				dh = gda_server_provider_get_data_handler_g_type (prov, context->cnc,
-										  G_VALUE_TYPE (expr->value));
-				if (!dh) goto err;
-			}
-			else
-				dh = gda_data_handler_get_default (G_VALUE_TYPE (expr->value));
-
-			if (dh)
-				str = gda_data_handler_get_sql_from_value (dh, expr->value);
-			else
-				str = gda_value_stringify (expr->value);
+		else {
+			str = gda_value_stringify (expr->value);
 			if (!str) goto err;
+			if (is_null && gda_value_is_null (expr->value))
+				*is_null = TRUE;
+			else if (is_default && (G_VALUE_TYPE (expr->value) == G_TYPE_STRING) && 
+				 !g_ascii_strcasecmp (g_value_get_string (expr->value), "default"))
+				*is_default = TRUE;
 		}
 	}
 	else if (expr->func) {
@@ -1581,33 +1430,8 @@ default_render_expr (GdaSqlExpr *expr, GdaSqlRenderingContext *context, gboolean
 		gchar *tmp;
 		tmp = context->render_operation (GDA_SQL_ANY_PART (expr->cond), context, error);
 		if (!tmp) goto err;
-		str = NULL;
-		if (GDA_SQL_ANY_PART (expr)->parent) {
-			if (GDA_SQL_ANY_PART (expr)->parent->type == GDA_SQL_ANY_STMT_SELECT) {
-				GdaSqlStatementSelect *selst;
-				selst = (GdaSqlStatementSelect*) (GDA_SQL_ANY_PART (expr)->parent);
-				if ((expr == selst->where_cond) ||
-				    (expr == selst->having_cond))
-					str = tmp;
-			}
-			else if (GDA_SQL_ANY_PART (expr)->parent->type == GDA_SQL_ANY_STMT_DELETE) {
-				GdaSqlStatementDelete *delst;
-				delst = (GdaSqlStatementDelete*) (GDA_SQL_ANY_PART (expr)->parent);
-				if (expr == delst->cond)
-					str = tmp;
-			}
-			else if (GDA_SQL_ANY_PART (expr)->parent->type == GDA_SQL_ANY_STMT_UPDATE) {
-				GdaSqlStatementUpdate *updst;
-				updst = (GdaSqlStatementUpdate*) (GDA_SQL_ANY_PART (expr)->parent);
-				if (expr == updst->cond)
-					str = tmp;
-			}
-		}
-
-		if (!str) {
-			str = g_strconcat ("(", tmp, ")", NULL);
-			g_free (tmp);
-		}
+		str = g_strconcat ("(", tmp, ")", NULL);
+		g_free (tmp);
 	}
 	else if (expr->select) {
 		gchar *str1;
@@ -1637,7 +1461,10 @@ default_render_expr (GdaSqlExpr *expr, GdaSqlRenderingContext *context, gboolean
 		str = g_strdup ("NULL");
 	}
 
-	if (!str) goto err;
+	if (!str) {
+		/* TO REMOVE */
+		str = g_strdup ("[...]");
+	}
 
 	if (expr->cast_as) 
 		g_string_append_printf (string, "CAST (%s AS %s)", str, expr->cast_as);
@@ -1729,11 +1556,7 @@ default_render_function (GdaSqlFunction *func, GdaSqlRenderingContext *context,
 			g_string_append (string, ", ");
 		str = context->render_expr (list->data, context, NULL, NULL, error);
 		if (!str) goto err;
-		if (((GdaSqlExpr*) list->data)->select)
-			g_string_append_c (string, '(');
 		g_string_append (string, str);
-		if (((GdaSqlExpr*) list->data)->select)
-			g_string_append_c (string, ')');
 		g_free (str);
 	}
 	g_string_append_c (string, ')');
@@ -1783,7 +1606,7 @@ default_render_operation (GdaSqlOperation *op, GdaSqlRenderingContext *context,
 			goto out;
 		}
 		sqlop->sql = str;
-		if (expr->case_s || expr->select)
+		if (expr->cond || expr->case_s || expr->select)
 			sqlop->is_composed = TRUE;
 		sql_list = g_slist_prepend (sql_list, sqlop);
 	}
@@ -2012,16 +1835,6 @@ default_render_case (GdaSqlCase *case_s, GdaSqlRenderingContext *context, GError
 	return NULL;
 }
 
-static gboolean
-alias_is_quoted (const gchar *alias)
-{
-	g_assert (alias);
-	if ((*alias == '\'') || (*alias == '"'))
-		return TRUE;
-	else
-		return FALSE;
-}
-
 static gchar *
 default_render_select_field (GdaSqlSelectField *field, GdaSqlRenderingContext *context, GError **error)
 {
@@ -2040,21 +1853,8 @@ default_render_select_field (GdaSqlSelectField *field, GdaSqlRenderingContext *c
 	g_string_append (string, str);
 	g_free (str);
 
-	if (field->as) {
-		if (! alias_is_quoted (field->as)) {
-			GdaConnectionOptions cncoptions = 0;
-			gchar *tmp;
-			if (context->cnc)
-				g_object_get (G_OBJECT (context->cnc), "options", &cncoptions, NULL);
-			tmp = gda_sql_identifier_quote (field->as, context->cnc,
-							context->provider, FALSE,
-							cncoptions & GDA_CONNECTION_OPTIONS_SQL_IDENTIFIERS_CASE_SENSITIVE);
-			g_string_append_printf (string, " AS %s", tmp);
-			g_free (tmp);
-		}
-		else
-			g_string_append_printf (string, " AS %s", field->as);
-	}
+	if (field->as)
+		g_string_append_printf (string, " AS %s", field->as);
 
 	str = string->str;
 	g_string_free (string, FALSE);
@@ -2070,7 +1870,6 @@ default_render_select_target (GdaSqlSelectTarget *target, GdaSqlRenderingContext
 {
 	GString *string;
 	gchar *str;
-	gpointer tmp;
 
 	g_return_val_if_fail (target, NULL);
 	g_return_val_if_fail (GDA_SQL_ANY_PART (target)->type == GDA_SQL_ANY_SQL_SELECT_TARGET, NULL);
@@ -2086,29 +1885,17 @@ default_render_select_target (GdaSqlSelectTarget *target, GdaSqlRenderingContext
 		g_free (str);
 	}
 	else {
+		gboolean tmp;
 		tmp = target->expr->value_is_ident;
-		target->expr->value_is_ident = (gpointer) 0x1;
+		target->expr->value_is_ident = TRUE;
 		str = context->render_expr (target->expr, context, NULL, NULL, error);
 		target->expr->value_is_ident = tmp;
 		string = g_string_new (str);
 		g_free (str);
 	}
 
-	if (target->as) {
-		if (! alias_is_quoted (target->as)) {
-			GdaConnectionOptions cncoptions = 0;
-			gchar *tmp;
-			if (context->cnc)
-				g_object_get (G_OBJECT (context->cnc), "options", &cncoptions, NULL);
-			tmp = gda_sql_identifier_quote (target->as, context->cnc,
-							context->provider, FALSE,
-							cncoptions & GDA_CONNECTION_OPTIONS_SQL_IDENTIFIERS_CASE_SENSITIVE);
-			g_string_append_printf (string, " AS %s", tmp);
-			g_free (tmp);
-		}
-		else
-			g_string_append_printf (string, " AS %s", target->as);
-	}
+	if (target->as)
+		g_string_append_printf (string, " AS %s", target->as);
 
 	str = string->str;
 	g_string_free (string, FALSE);
