@@ -99,6 +99,8 @@ data_widget_init (DataWidget *dwid, G_GNUC_UNUSED DataWidgetClass *klass)
 	/* allocate private structure */
 	dwid->priv = g_new0 (DataWidgetPrivate, 1);
 
+	gtk_orientable_set_orientation (GTK_ORIENTABLE (dwid), GTK_ORIENTATION_VERTICAL);
+
 	/* init Widgets's structure */
 	dwid->priv->top_nb = gtk_notebook_new ();
 	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (dwid->priv->top_nb), FALSE);
@@ -117,7 +119,7 @@ data_widget_init (DataWidget *dwid, G_GNUC_UNUSED DataWidgetClass *klass)
 
 	/* contents page */
 	GtkWidget *vbox;
-	vbox = gtk_vbox_new (FALSE, 0);
+	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 	gtk_notebook_append_page (GTK_NOTEBOOK (dwid->priv->top_nb), vbox, NULL);
 	dwid->priv->contents_page_vbox = vbox;
 
@@ -142,7 +144,7 @@ data_widget_get_type (void)
 			(GInstanceInitFunc) data_widget_init,
 			0
 		};
-		type = g_type_register_static (GTK_TYPE_VBOX, "DataWidget", &info, 0);
+		type = g_type_register_static (GTK_TYPE_BOX, "DataWidget", &info, 0);
 	}
 	return type;
 }
@@ -208,14 +210,14 @@ create_or_reuse_part (DataWidget *dwid, DataSource *source, gboolean *out_reused
 	dwid->priv->parts = g_slist_append (dwid->priv->parts, part);
 
 	GtkWidget *vbox;
-	vbox = gtk_vbox_new (FALSE, 0);
+	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 	part->top = vbox;
 	g_object_ref_sink ((GObject*) part->top);
 
 	GtkWidget *header, *label, *button, *image;
 	const gchar *cstr;
 
-	header = gtk_hbox_new (FALSE, 0);
+	header = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
 	gtk_box_pack_start (GTK_BOX (vbox), header, FALSE, FALSE, 0);
 
 	label = gtk_label_new ("");
@@ -278,12 +280,12 @@ make_paned_list (gint size, gboolean horiz)
 	GtkWidget *paned;
 
 	g_assert (size >= 2);
-	paned = horiz ? gtk_hpaned_new () : gtk_vpaned_new ();
+	paned = gtk_paned_new (horiz ? GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL);
 	list = g_slist_prepend (list, paned);
 
 	for (i = 2; i < size; i++) {
 		GtkWidget *paned2;
-		paned2 = horiz ? gtk_hpaned_new () : gtk_vpaned_new ();
+		paned2 = gtk_paned_new (horiz ? GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL);
 		gtk_paned_add2 (GTK_PANED (paned), paned2);
 		list = g_slist_prepend (list, paned2);
 		paned = paned2;
@@ -371,7 +373,7 @@ update_layout (DataWidget *dwid)
 	GError *lerror = NULL;
 	GtkWidget *new_contents;
 
-	new_contents = gtk_vbox_new (FALSE, 0);
+	new_contents = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 
 	sources_array = data_source_manager_get_sources_array (dwid->priv->mgr, &lerror);
 	if (!sources_array) {
@@ -740,7 +742,7 @@ source_exec_finished_cb (G_GNUC_UNUSED DataSource *source, GError *error, DataPa
 #endif
 		
 		/* compute part->export_data */
-		formgrid_data_set_changed_cb (cwid, part);
+		formgrid_data_set_changed_cb (UI_FORMGRID (cwid), part);
 	}
 	else {
 		GError *lerror = NULL;
