@@ -36,8 +36,6 @@ static void query_exec_perspective_init (QueryExecPerspective *stmt);
 static void query_exec_perspective_dispose (GObject *object);
 
 static void query_exec_perspective_grab_focus (GtkWidget *widget);
-static void query_exec_add_cb (G_GNUC_UNUSED GtkAction *action, BrowserPerspective *bpers);
-
 
 /* BrowserPerspective interface */
 static void                 query_exec_perspective_perspective_init (BrowserPerspectiveIface *iface);
@@ -89,7 +87,7 @@ query_exec_perspective_get_type (void)
 		
 		g_static_mutex_lock (&registering);
 		if (type == 0) {
-			type = g_type_register_static (GTK_TYPE_VBOX, "QueryExecPerspective", &info, 0);
+			type = g_type_register_static (GTK_TYPE_BOX, "QueryExecPerspective", &info, 0);
 			g_type_add_interface_static (type, BROWSER_PERSPECTIVE_TYPE, &perspective_info);
 		}
 		g_static_mutex_unlock (&registering);
@@ -131,6 +129,8 @@ query_exec_perspective_init (QueryExecPerspective *perspective)
 {
 	perspective->priv = g_new0 (QueryExecPerspectivePrivate, 1);
 	perspective->priv->favorites_shown = TRUE;
+
+	gtk_orientable_set_orientation (GTK_ORIENTABLE (perspective), GTK_ORIENTATION_VERTICAL);
 }
 
 static void fav_selection_changed_cb (GtkWidget *widget, gint fav_id, BrowserFavoritesType fav_type,
@@ -160,7 +160,7 @@ query_exec_perspective_new (BrowserWindow *bwin)
 
 	/* contents */
 	GtkWidget *paned, *nb, *wid;
-	paned = gtk_hpaned_new ();
+	paned = gtk_paned_new (GTK_ORIENTATION_HORIZONTAL);
 	if (fav_supported) {
 		wid = query_favorite_selector_new (bcnc);
 		g_signal_connect (wid, "selection-changed",
@@ -186,7 +186,7 @@ query_exec_perspective_new (BrowserWindow *bwin)
 	gtk_notebook_append_page (GTK_NOTEBOOK (nb), page, tlabel);
 
 	gtk_notebook_set_tab_reorderable (GTK_NOTEBOOK (nb), page, TRUE);
-	gtk_notebook_set_group (GTK_NOTEBOOK (nb), bcnc + 0x01); /* add 0x01 to differentiate it from SchemaBrowser */
+	gtk_notebook_set_group_name (GTK_NOTEBOOK (nb), "query-exec");
 	gtk_notebook_set_tab_detachable (GTK_NOTEBOOK (perspective->priv->notebook), page,
 					 TRUE);
 
@@ -215,12 +215,8 @@ fav_selection_changed_cb (G_GNUC_UNUSED GtkWidget *widget, gint fav_id,
 
 	nb = GTK_NOTEBOOK (perspective->priv->notebook);
 	page = gtk_notebook_get_nth_page (nb, gtk_notebook_get_current_page (nb));
-	if (!page) {
-		query_exec_add_cb (NULL, BROWSER_PERSPECTIVE (perspective));
-		page = gtk_notebook_get_nth_page (nb, gtk_notebook_get_current_page (nb));
-		if (!page)
-			return;
-	}
+	if (!page)
+		return;
 	if (IS_QUERY_CONSOLE_PAGE_PAGE (page)) {
 		query_console_page_set_text (QUERY_CONSOLE_PAGE (page), selection, fav_id);
 		gtk_widget_grab_focus (page);
